#!/bin/sh
#
# This script will test following device on ATC300 for production.
#  PIC
#  Modem and RS485
#

################################################################################
# Enter Operator ID
################################################################################
get_operator_id()
{
    echo
    echo "Enter your Operator Info: "
    read OPERTATOR_ID
    echo "ATC300 testing performed by: $OPERTATOR_ID" >> /var/log/atc300_test.log
}

################################################################################
# Enter Serial Number
# serial number will be provided at installation and saved to /Serial.txt.
################################################################################
get_serial_num()
{
    if [ -f /Serial.txt ]; then
        SERIAL_NUM=`cat /Serial.txt`

        echo
        echo "Found ATC300 serial number: $SERIAL_NUM"
        echo "Change ATC300 serial number."
        echo "Press y <ENTER> or n <ENTER>."
        read TEST_RESULT
        if [ "$TEST_RESULT" = "y" ];then
            echo
            echo "Enter new ATC300 serial number: "
            read SERIAL_NUM
            echo "New ATC300 serial number: $SERIAL_NUM" >> /var/log/atc300_test.log
            echo "$SERIAL_NUM" > /Serial.txt
        else
            echo
            echo "ATC300 serial number: $SERIAL_NUM" >> /var/log/atc300_test.log
        fi
    else
        echo
        echo "No ATC300 serial number found." >> /var/log/atc300_test.log
        echo "Serial number should have been created at install." >> /var/log/atc300_test.log
        echo "Creating new serial number." >> /var/log/atc300_test.log
        echo
        echo "Enter ATC300 serial number: "
        read SERIAL_NUM
        echo "ATC300 serial number: $SERIAL_NUM" >> /var/log/atc300_test.log
        echo "$SERIAL_NUM" > /Serial.txt
        echo
    fi
}

################################################################################
# Run PIC test
#  Checks the Relay 1, Relay 2 and POWER_ON.

# Power test moved from "atc300_test.sh" to "atc300_install.sh" due to
#  tilts running right after power test.  Actuator need at minimum 30
#  seconds to reinitialize (ENABLED), this delay is much too long for
#  for production.  Imtermittent error "DEVICE_DISABLED" will occur if
#  tilt is run before actuator are reinitialized.
################################################################################
test_pic()
{
    echo
    echo " Starting PIC test........." >> /var/log/atc300_test.log
    echo

    # Relay 1 test
    /usr/bin/pictestapp 1 1 2>&1 | tee -a /var/log/atc300_test.log
    echo
    echo "Verify K1 LED is ON (Green)."
    echo "Press y <ENTER> or n <ENTER>."
    read TEST_RESULT
    if [ "$TEST_RESULT" = "y" ];then
        echo "Relay 1 ON test PASSED." >> /var/log/atc300_test.log
    else
        echo "Relay 1 ON test FAILED." >> /var/log/atc300_test.log
        exit 1
    fi
    /usr/bin/pictestapp 1 0 2>&1 | tee -a /var/log/atc300_test.log
    echo
    echo "Verify K1 LED is OFF."
    echo "Press y <ENTER> or n <ENTER>."
    read TEST_RESULT
    if [ "$TEST_RESULT" = "y" ];then
        echo "Relay 1 OFF test PASSED." >> /var/log/atc300_test.log
    else
        echo "Relay 1 OFF test FAILED." >> /var/log/atc300_test.log
        exit 1
    fi

    # Relay 2 test
    /usr/bin/pictestapp 2 1 2>&1 | tee -a /var/log/atc300_test.log
    echo
    echo "Verify K2 LED is ON (Green)."
    echo "Press y <ENTER> or n <ENTER>."
    read TEST_RESULT
    if [ "$TEST_RESULT" = "y" ];then
        echo "Relay 2 ON test PASSED." >> /var/log/atc300_test.log
    else
        echo "Relay 2 ON test FAILED." >> /var/log/atc300_test.log
        exit 1
    fi
    /usr/bin/pictestapp 2 0 2>&1 | tee -a /var/log/atc300_test.log
    echo
    echo "Verify K2 LED is OFF."
    echo "Press y <ENTER> or n <ENTER>."
    read TEST_RESULT
    if [ "$TEST_RESULT" = "y" ];then
        echo "Relay 2 OFF test PASSED." >> /var/log/atc300_test.log
    else
        echo "Relay 2 OFF test FAILED." >> /var/log/atc300_test.log
        exit 1
    fi
}


################################################################################
# Run device scan to test functionality and build database
################################################################################
test_device_scan()
{
    echo " Starting device scan. This may take a few minutes."
    echo " Starting device scan........" >> /var/log/atc300_test.log
    if [ -f /var/ascu/Handle.db ]; then
      rm -f /var/ascu/Handle.db
    fi
    echo 7 >/tmp/scanmax.txt
    if [ -e /tmp/cancelscan.txt ]; then
      rm -f /tmp/cancelscan.txt
    fi
    if [ -e /tmp/scanclue.txt ]; then
      rm -f /tmp/scanclue.txt
    fi
    export QUERY_STRING="action=Send+One+Shot+Scan&scan=Auto-scan"
    /usr/bin/aisg_command >/dev/null &
    sleep 5

    COUNT=0
    while [ $COUNT -lt 20 ]; do
      if [ -f /tmp/devscanstarted.txt ]; then
        COUNT=20
      else
        sleep 1
        let COUNT=$COUNT+1
      fi
    done
    FOUND=0

    while [ -f /tmp/devscanstarted.txt ]; do
      echo . 
      sleep 6
    done
    if [ -f /var/ascu/Handle.db ]; then
      FOUND=`cat /var/ascu/Handle.db | wc -l`
    fi
    if [ "$FOUND" -eq 7 ]; then
      echo "Device scan test PASSED." >> /var/log/atc300_test.log
    else
      echo "Not all devices found.  Device scan test FAILED."
      echo "Device scan test FAILED." >> /var/log/atc300_test.log
      echo "Deleting device database."
      /usr/bin/delall.sh >/dev/null &
      exit 1
    fi
}

################################################################################
# Run tilt test #1 on all actuator once.
################################################################################
test_tilt()
{
    echo
    echo " Starting actuator tilt test........" >> /var/log/atc300_test.log
    echo
    /usr/bin/atc300_tilt.sh 1 0 1 2>&1 | tee -a /var/log/atc300_test.log
}


#
#Start main procedure
#

get_operator_id
get_serial_num
test_pic
test_device_scan
test_tilt
