#!/bin/sh
makedb () {
# Create default site DB
ascu_db_trans -db Site.db -w "0	$1"
ascu_db_trans -db Site.db -w "1	RET1 ASCU"
ascu_db_trans -db Site.db -w "2	$2"
ascu_db_trans -db Site.db -w "3	$3"
ascu_db_trans -db Site.db -w "4	0"
ascu_db_trans -db Site.db -w "5	300"
ascu_db_trans -db Site.db -w "6	$4"
ascu_db_trans -db Site.db -w "7	0"
ascu_db_trans -db Site.db -w "9	$5"
ascu_db_trans -db Site.db -w "12	0"
}
qt () { "$@" >/dev/null 2>&1 ; }

#init hour counter
if [ -f /var/run/ISTI ]; then
	HOURS=`cat /var/run/ISTI`
else
	HOURS=0
	echo $HOURS >/var/run/ISTI
fi
cd /var/ascu
DBVERS=`ascu_db_trans -v`   #This returns a hardcoded enum

# if we're booting up, don't exit due to old reboot cmd!
if [ -f /var/run/hcexit ]; then
	rm /var/run/hcexit
fi

# extract DB from archive if not present
if [ ! -f /var/ascu/Site.db ]; then
   if [ ! -f /var/ascu/ascudb.tar ]; then
   	if [ -f /tmp/ascudb.tar ]; then
         mv /tmp/ascudb.tar /var/ascu/ascudb.tar
      fi
   fi

	if [ -f /var/ascu/ascudb.tar ]; then
		if [ -f Error.db ]; then
			mv Error.db Error.new
		fi
		if [ -f snmpd2.conf ]; then
			mv snmpd2.conf snmpd2.conf.new
		fi
		tar xf /var/ascu/ascudb.tar
		rm /var/ascu/ascudb.tar
		if [ -f Error.new ]; then
			mv Error.new Error.db
		fi
		if [ -f snmpd2.conf.new ]; then
			mv snmpd2.conf.new snmpd2.conf
		fi
	elif [ -f /tmp/ascudb.tar ]; then
                echo >>/dev/null
	else
		# 1st time we've ever run
		echo First time to run
		if [ -z "$MDATE" ]; then
                        #set MDATE with current date/time
			#MDATE=`date +%Y.%m.%d-%H:%M:%S`
			MDATE=`date +%s`
			echo Manufacture date is $MDATE
		fi
                if [ ! -f /Serial.txt ]; then
                  SERIAL=00000
                else
                  SERIAL=`cat /Serial.txt`
                fi

        # retrieve IP address and net mask.
        IPADDR=`ifconfig eth0 |grep "inet addr:" |cut -d ':' -f 2 |cut -d ' ' -f 1`
        NETMASK=`ifconfig eth0 |grep "inet addr:" |cut -d ':' -f 4`
		makedb $DBVERS $SERIAL $MDATE $IPADDR $NETMASK
		echo Created /var/ascu/Site.db
	fi
fi

LOGFILES="messages snmpd.log upload cron exedev"
MINS=0
DELTA=`date +%s`
SKEW=0

# Remove web password when upgrading to Site Sharing
if [ -f /var/ascu/uploads/prev_version ]; then
  PREV_VERSION=`cat /var/ascu/uploads/prev_version`
  SS_VERSION="2.36_A"
  if [ "$PREV_VERSION" \< "$SS_VERSION" ]; then
    if [ -f /password.txt ]; then
      rm /password.txt
      rm /var/ascu/uploads/prev_version
    fi
  fi
fi

# Main housekeeping loop
while true ; do
	if [ -f /var/run/hcexit ]; then
                rm /var/run/hcexit
		exit 0
	fi

    if [ -f /tmp/updt_hwclock.txt ]; then
        /sbin/hwclock -w    # set hardware clock to the system time
        rm /tmp/updt_hwclock.txt
    fi

	NOW=`date +%s`
	if [ "$MINS" -ge 59 ]; then
		# Handle ISTI increment
		MINS=0
		let HOURS=$HOURS+1
		echo $HOURS >/var/run/ISTI
		# Try to eliminate accumulated error
		if [ "$SKEW" = 0 ]; then
			let DELTA=$DELTA+3600
			let WAIT=$DELTA-$NOW
			if [ $WAIT -gt 60 ]; then
				WAIT=45
			fi
		else
			WAIT=45
			SKEW=0
		fi
		if [ $WAIT -gt 0 ]; then
			sleep $WAIT
		fi
		NOW=`date +%s`
		DELTA=$NOW
	fi
	if [ -f /var/run/Dirty.db ]; then
		# Handle DB writes
		rm /var/run/Dirty.db
		SNMP=""
		if [ -f snmpd2.conf ]; then
			SNMP="snmpd2.conf"
		fi
		if [ -f Error.db ]; then
			mv Error.db Error.new
		fi
		#tar cf /tmp/ascudb.tar *.db *.tag $SNMP
		tar cf /var/ascu/ascudb.tar *.db *.tag $SNMP
		if [ -f Error.new ]; then
			mv Error.new Error.db
		fi
	fi
	# truncate log files
	if [ -f /var/run/syslogd.pid ]; then
		for FILE in $LOGFILES ; do
			if [ -f /var/log/"$FILE" ]; then
				SIZE=`wc -l /var/log/$FILE`
				if [ `echo $SIZE | cut -d ' ' -f 1` -gt 500 ]; then
					if [ -f /var/log/$FILE.2 ]; then
						rm /var/log/$FILE.2
					fi
					if [ -f /var/log/$FILE.1 ]; then
						mv /var/log/$FILE.1 /var/log/$FILE.2
					fi
					mv /var/log/$FILE /var/log/$FILE.1
					kill -1 `cat /var/run/syslogd.pid`
				fi
			fi
		done
	else
		# no syslog? restart it
		/sbin/syslogd -n &
		echo $! >/var/run/syslogd.pid
	fi
	if [ -f /var/run/runme ]; then
		RUNME=`cat /var/run/runme`
		rm /var/run/runme
		$RUNME &
		echo Run $RUNME from /var/run/runme
	fi
	# Add any extra commands here
	# Total time iterating loop must be < 60 seconds
	# Reboot must be last
	if [ ! -f /var/lock/boot ]; then
		if [ -f /var/run/reboot ]; then
			# set LED to red
			mv /var/run/reboot /var/run/boot-me-now
			setstat -d 0 -s 32768
			sleep 6
			echo Do reboot
		fi
	fi

	# try to minimise errors
	let LATENCY=`date +%s`-$NOW
	if [ "$LATENCY" -gt 60 -o "$LATENCY" -lt 0 ]; then
		# assume date has been adjusted
		SKEW=$LATENCY
		LATENCY=0
	fi
	let ZZZ="60-$LATENCY"
	if [ $ZZZ -gt 60 ]; then
		ZZZ=59
	fi
	while [ $ZZZ -gt 15 ]; do
		STIME=`date +%s`
		sleep 10
		if [ -f /var/run/runme ]; then
			RUNME=`cat /var/run/runme`
			rm /var/run/runme
			$RUNME &
			echo Run $RUNME from /var/run/runme
		fi
		let LATENCY=`date +%s`-$STIME
		let ZZZ="$ZZZ-$LATENCY"
	done
	sleep $ZZZ
	let MINS=$MINS+1

        # check for scheduled rehoming
	if [ -f /RehomeDate.txt ]; then
		if [ -f /RehomeIP.txt ]; then
			REHOME_DATE=`cat /RehomeDate.txt`
			STIME=`date +%s`
			if [ $STIME -ge $REHOME_DATE ]; then
                		if [ ! -f /RehomeState.txt ]; then
					# enter rehoming with initial rehoming state
					echo "111,$STIME" >/RehomeState.txt

					# configure the Original IP files
					if [ -f /OrigIP.txt ]; then
						rm /OrigIP.txt
					fi
					if [ -f /StaticIP.txt ]; then
						mv /StaticIP.txt /OrigIP.txt
					else
						echo "DHCP" >/OrigIP.txt
					fi
					if [ -f /OrigNetMask.txt ]; then
						rm /OrigNetMask.txt
					fi
					if [ -f /NetMask.txt ]; then
						mv /NetMask.txt /OrigNetMask.txt
					fi
					if [ -f /OrigRoute.txt ]; then
						rm /OrigRoute.txt
					fi
					if [ -f /Route.txt ]; then
						mv /Route.txt /OrigRoute.txt
					fi
				else # prep for rehoming cfg to be moved in
					if [ -f /StaticIP.txt ]; then
						rm /StaticIP.txt
					fi
					if [ -f /NetMask.txt ]; then
						rm /NetMask.txt
					fi
					if [ -f /Route.txt ]; then
						rm /Route.txt
					fi
				fi
				# use the rehoming IP address on next boot
				REHOME_IP=`cat /RehomeIP.txt`
				cp /RehomeIP.txt /StaticIP.txt
				ascu_db_trans -db Site.db -f 6 -w "6	$REHOME_IP"
				if [ -f /RehomeNetmask.txt ]; then
					REHOME_NETMASK=`cat /RehomeNetmask.txt`
					cp /RehomeNetmask.txt /NetMask.txt
					ascu_db_trans -db Site.db -f 9 -w "9	$REHOME_NETMASK"
				fi
				if [ -f /RehomeIPRoute.txt ]; then
					REHOME_ROUTER=`cat /RehomeIPRoute.txt`
					cp /RehomeIPRoute.txt /Route.txt
					ascu_db_trans -db Site.db -f 10 -w "10	$REHOME_ROUTER"
				fi
				# remove the rehoming schedule file
				if [ -f /RehomeDate.txt ]; then
					rm /RehomeDate.txt
				fi
				# restart the controller for the IP change to take effect
				echo Restarting with rehoming IP configuration
				reboot
			fi
		fi
	elif [ -f /OrigCfgDate.txt ]; then
		if [ -f /OrigIP.txt ]; then
			REHOME_DATE=`cat /OrigCfgDate.txt`
			STIME=`date +%s`
			if [ $STIME -ge $REHOME_DATE ]; then
				if [ -f /StaticIP.txt ]; then
					rm /StaticIP.txt
				fi
				if [ -f /NetMask.txt ]; then
					rm /NetMask.txt
				fi
				if [ -f /Route.txt ]; then
					rm /Route.txt
				fi
				# use the original IP address on next boot
				REHOME_IP=`cat /OrigIP.txt`
				if [ "$REHOME_IP" != "DHCP" ]; then
					cp /OrigIP.txt /StaticIP.txt
					ascu_db_trans -db Site.db -f 6 -w "6	$REHOME_IP"
				fi
				if [ -f /OrigNetMask.txt ]; then
					REHOME_NETMASK=`cat /OrigNetMask.txt`
					cp /OrigNetMask.txt /NetMask.txt
					ascu_db_trans -db Site.db -f 9 -w "9	$REHOME_NETMASK"
				fi
				if [ -f /OrigRoute.txt ]; then
					REHOME_ROUTER=`cat /OrigRoute.txt`
					cp /OrigRoute.txt /Route.txt
					ascu_db_trans -db Site.db -f 10 -w "10	$REHOME_ROUTER"
				fi
				# remove the rehoming schedule file
				if [ -f /OrigCfgDate.txt ]; then
					rm /OrigCfgDate.txt
				fi
				# restart the controller for the IP change to take effect
				echo Restarting with original IP configuration
				reboot
			fi
		fi
	fi

	if [ -f /var/ascu/CSDirty.txt ]; then
		# The site checksum requires recalculation
		rm /var/ascu/CSDirty.txt
		ascu_db_trans -c
	fi

        # check for process that's grown too big
	VMLIMIT=4000
	MYSIZE=`cat /proc/$$/status | grep VmSize | cut -b '12-16'`
	if [ "$MYSIZE" -gt "$VMLIMIT" ]; then 
		echo this process has gotten too big
		reboot
	fi
done
