#!/bin/sh -xv

# a file is kept in ramdisk IP_FILE
# each time a web client access is attempted
# a line is added to the file of format
# IP_address time_stamp

# define files, paths and timeout in seconds 
TIMEOUT=180
PASSWORD_FILE=/password.txt
TEMP_FILE=/tmp/mytmp
IP_FILE=/tmp/RemoteAddr.txt
RECOVERY_PASSWORD_FILE=/Serial.txt

GetOperatorName()
{
# init return value
retval=1

CheckPasswordAging

# determine number of lines in access request file
i=1
if [ -f $IP_FILE ]
then
	end=`/usr/bin/wc $IP_FILE | /usr/bin/cut -d \  -f5,6,7` 
else
	logOper=""
	end=0
fi

# go through file, find this IP address and operator name  
while [ "$i" -le "$end" ] && [ $retval -eq "1" ]
do
	line=`head -n $i $IP_FILE | tail -1`
	logdate=`echo $line | /usr/bin/cut -d \  -f2` 
	logIP=`echo $line | /usr/bin/cut -d \  -f1` 
	if test "$REMOTE_ADDR" = "$logIP"
	then
		logOper=`echo $line | /usr/bin/cut -d \  -f3`
		retval=0
	fi	
	i=`expr $i + 1`
done

if [ -f $PASSWORD_FILE ] && [ $retval -eq "1" ]
then
	logOper="99"
fi
if [ -z "$logOper" ] || [ "$logOper" = "10" ]; then
	return 1
else
	return 0
fi
}

CheckPasswordAging()
{
# init return value
retval=1

# remove temp file if present
if [ -f $TEMP_FILE ]
then
	/bin/rm -f $TEMP_FILE
fi	

# get time in seconds since beginning of Linux
currdate=`date -u '+%s'`

# set expiration date 
expiredate=`expr $currdate - $TIMEOUT`

# determine number of lines in access request file
i=1
if [ -f $IP_FILE ]
then
	end=`/usr/bin/wc $IP_FILE | /usr/bin/cut -d \  -f5,6,7` 
else
	end=0
fi

# go through file, copy unexpired accesses to temp file  
while [ "$i" -le "$end" ]
do
	line=`head -n $i $IP_FILE | tail -1`
	logdate=`echo $line | /usr/bin/cut -d \  -f2` 
	logIP=`echo $line | /usr/bin/cut -d \  -f1` 
	logOperTmp=`echo $line | /usr/bin/cut -d \  -f3`
	if [ "$logdate" -ge "$expiredate" ]
	then
		if test "$REMOTE_ADDR" = "$logIP"
		then
			logOper=$logOperTmp
			echo $REMOTE_ADDR $currdate $logOperTmp >>$TEMP_FILE
			retval=0
		else	
			echo "$logIP $logdate $logOperTmp" >>$TEMP_FILE
		fi
	fi	
	i=`expr $i + 1`
done

# move temp file to access file, now have eliminated expired stuff
if [ -f $TEMP_FILE ]
then
	/bin/mv $TEMP_FILE $IP_FILE
elif [ -f $IP_FILE ]
then
	/bin/rm -f $IP_FILE
fi

return $retval

}

GetRecoveryPassword()
{
# This generates a recovery or rescue password if the user
# forgets his password to the web interface
# The algorithm uses the system serial number and works like this:
# ( ((Digit 2-6) * (Digit 7-10)) concatenate 
#		with  ((Digit 7-10) * (Digit 7-10)) )
# Then take all of this and if longer than 8 digits
# Truncate to the lower 8 digits then divide by 7
#
# Example: Serial number FV98765432
# (9876*5432) concatenate with (5432*5432)
# which yields
# 5364643229506624
# Truncate to 29506624
# Divide by 7 and password is: 4215232

RecoveryPasswordBase=`cat /Serial.txt`
RPUpper=`expr substr $RecoveryPasswordBase 3 4`
RPLower=`expr substr $RecoveryPasswordBase 7 4`

RP1=`expr $RPLower '*' $RPUpper`
RP2=`expr $RPLower '*' $RPLower`
RP12=`expr $RP1$RP2`

len=`expr length "$RP12"`
if test "$len" -gt 8
then
	Extra=`expr "$len" - 7`
	RPout=`expr substr $RP12 $Extra 8`
else
	RPout=$RP12
	
fi

RecoveryPassword=`expr $RPout '/' 7`

}

TestPassword()
{
	GetRecoveryPassword
	if test "$EnteredPassword" = "$RecoveryPassword"
	then
		rm $PASSWORD_FILE
		return 0
	fi
	AdminPassword=`cat $PASSWORD_FILE`
	EnteredPasswordHash=`echo $EnteredPassword | /usr/bin/md5sum | /usr/bin/cut -d \  -f1`
	if test "$AdminPassword" = "$EnteredPasswordHash"
	then
		currdate=`date -u '+%s'`
		echo "$REMOTE_ADDR $currdate 10" >>$IP_FILE
		return 0
 	else	
		EnteredPasswordLen=`expr length "$EnteredPassword"`
		if [ "$EnteredPasswordLen" -gt 0 ]
		then
			OPNAME1=`ascu_db_trans -db Opname.db -f 1 | cut -f 2`
			OPPW1=`ascu_db_trans -db Opname.db -f 1 | cut -f 3`
			OPNAME2=`ascu_db_trans -db Opname.db -f 2 | cut -f 2`
			OPPW2=`ascu_db_trans -db Opname.db -f 2 | cut -f 3`
			OPNAME3=`ascu_db_trans -db Opname.db -f 3 | cut -f 2`
			OPPW3=`ascu_db_trans -db Opname.db -f 3 | cut -f 3`
			OPNAME4=`ascu_db_trans -db Opname.db -f 4 | cut -f 2`
			OPPW4=`ascu_db_trans -db Opname.db -f 4 | cut -f 3`
#			if test "$EnteredPasswordHash" = "$OPPW1"
			if test "$EnteredPassword" = "$OPPW1"
			then
				currdate=`date -u '+%s'`
				echo "$REMOTE_ADDR $currdate $OPNAME1" >>$IP_FILE
				return 0
#			elif test "$EnteredPasswordHash" = "$OPPW2"
			elif test "$EnteredPassword" = "$OPPW2"
			then
				currdate=`date -u '+%s'`
				echo "$REMOTE_ADDR $currdate $OPNAME2" >>$IP_FILE
				return 0
#			elif test "$EnteredPasswordHash" = "$OPPW3"
			elif test "$EnteredPassword" = "$OPPW3"
			then
				currdate=`date -u '+%s'`
				echo "$REMOTE_ADDR $currdate $OPNAME3" >>$IP_FILE
				return 0
#			elif test "$EnteredPasswordHash" = "$OPPW4"
			elif test "$EnteredPassword" = "$OPPW4"
			then
				currdate=`date -u '+%s'`
				echo "$REMOTE_ADDR $currdate $OPNAME4" >>$IP_FILE
				return 0
			fi
		fi
 		return 1
	fi
}

PasswordActive()
{
	if [ ! -f $PASSWORD_FILE ]
	then
		return 1
	fi	
	
	Password=`cat $PASSWORD_FILE`
	if test $Password
	then
		return 0
	else
		return 1
	fi	
}


LogInLogic()
{
# START of password stuff

# first check if we have password access enabled
# if contents of password file is null, then it's disabled
PasswordActive
if test "$?" = 0
then
# if enabled, check if login has timed out
# if so, force log in required again
	CheckPasswordAging
	PASSWORD_REQUIRED="$?"
else
	PASSWORD_REQUIRED=0
fi	
#
# check if password has been entered and if it's correct 
#

EnteredPassword=""

# we use method GET when it's ok to send response in the URL
# however, here we are passing the password back, so we used POST
# the following is required to grab the POST response (which comes
# back in STDIN
if test "$CONTENT_LENGTH" -gt 0 
then
	read LINE
# echo "Line is {$LINE}" >/www/cgi-bin/debugfile
	EnteredPassword=`echo $LINE | /usr/bin/cut -d \& -f1 | /usr/bin/cut -d = -f2`
# echo  "EnteredPassword={$EnteredPassword}" >>/www/cgi-bin/debugfile

fi

# echo  "EnteredPassword={$EnteredPassword}" >>/www/cgi-bin/debugfile
 
EnteredPasswordLen=`expr length "$EnteredPassword"`
EnteredPasswordANLen=`expr "$EnteredPassword" : '[A-Za-z0-9]*'`
		
# echo  "EnteredPassword={$EnteredPassword}" >>/www/cgi-bin/debugfile
# echo  "EnteredPasswordLen=$EnteredPasswordLen" >>/www/cgi-bin/debugfile
# echo  "EnteredPasswordANLen=$EnteredPasswordANLen" >>/www/cgi-bin/debugfile

if [ "$EnteredPasswordLen" -ne "$EnteredPasswordANLen" ]
then
    let EnteredPasswordLen=$EnteredPasswordLen-1
    EnteredPassword=`echo $EnteredPassword | /usr/bin/cut -b1-$EnteredPasswordLen`
# echo  "Altered EnteredPassword={$EnteredPassword}" >>/www/cgi-bin/debugfile
fi

if test "$EnteredPasswordLen" -gt 0
then
	TestPassword
	if test "$?" = 0
	then
		PASSWORD_REQUIRED=0

	fi
			
fi
	
# finally,  if necessary, require password entry
if test "$PASSWORD_REQUIRED" = 1
then

	echo "<BR><form action=\"$PHP_SELF\" method=POST >"
	
        echo "<table width=\"99%\"  class=\"form_2\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">"
  	echo "<tr id=\"title\">"
    	echo "<th colspan=3>"
	echo "<label><span class=\"style1\">Security</span></label>"
        echo "&nbsp;"
    	echo "</th>"
        
        echo "<tr>"
        echo "<td width=\"30%\"><div align=\"right\">"
	echo "For access, please enter password:"
        echo "</div></td>"
        echo "<td width=\"30%\"><div align=\"left\">"
	echo "<input type=password name=EnteredPassword value=\"\">"
        echo "</div></td>"
        echo "</tr><tr><BR>"
        echo "</tr><tr>"
	
        echo "<td width=\"30%\"><div align=\"right\">"
        echo "<BR></div></td>"
        
        echo "<td width=\"30%\"><div align=\"left\">"
	echo "<input type=submit name=PasswordGoButton value=\"Log In\">"
        echo "</div></td></tr>"
	echo "</form>"
	echo "<tr>"
        echo "<td width=\"30%\"><div align=\"right\">"
        echo "<BR></div></td>"
        
	echo "</tr></table>"
	echo "</head>"
	
#	echo "For access, please enter password"
#	echo "<form action=\"$PHP_SELF\" method=POST>"
#	echo "<input type=password name=EnteredPassword value=\"\">"
#	echo "<input type=submit name=PasswordGoButton value=\"Log In\">"
#	echo "</form>"
#	echo "</head>"
	exit
fi
# END of password stuff
cat << EOF
EOF

}


DisablePassword()
{
	if [ -f $PASSWORD_FILE ]
	then
		/bin/rm -f $PASSWORD_FILE
	fi
	
	/bin/touch $PASSWORD_FILE
}

ChangePassword()
{
	if [ -f $PASSWORD_FILE ]
	then
		/bin/rm -f $PASSWORD_FILE
	fi
#	echo $PASSWORD >$PASSWORD_FILE
	echo $PASSWORD | /usr/bin/md5sum | /usr/bin/cut -d \  -f1 > $PASSWORD_FILE

}

ConfigurePasswordButtons()
{
PasswordActive
if test "$?" = 0
then
	ENABLE_PASSWORD_BUTTON="disabled=\"disabled\""
	DISABLE_PASSWORD_BUTTON=""
else
	ENABLE_PASSWORD_BUTTON=""
	DISABLE_PASSWORD_BUTTON="disabled=\"disabled\""
fi
}


LogOut()
{

# remove temp file if present
if [ -f $TEMP_FILE ]
then
	/bin/rm -f $TEMP_FILE
fi	

# determine number of lines in access request file
i=1
if [ -f $IP_FILE ]
then
	end=`/usr/bin/wc $IP_FILE | /usr/bin/cut -d \  -f5,6,7` 
else
	end=0
fi

# go through file, copy unexpired accesses to temp file  
while [ "$i" -le "$end" ]
do
	line=`head -n $i $IP_FILE | tail -1`
	logdate=`echo $line | /usr/bin/cut -d \  -f2` 
	logIP=`echo $line | /usr/bin/cut -d \  -f1` 
	logOperTmp=`echo $line | /usr/bin/cut -d \  -f3`
	if test "$REMOTE_ADDR" != "$logIP"
	then
		echo "$logIP $logdate $logOperTmp" >>$TEMP_FILE
	fi	
	i=`expr $i + 1`
done

# move temp file to access file, now have eliminated expired stuff
if [ -f $TEMP_FILE ]
then
	/bin/mv $TEMP_FILE $IP_FILE
elif [ -f $IP_FILE ]
then
	/bin/rm -f $IP_FILE
fi
logOper=""

}



PrettyMessage()
{

	echo "<BR><form action=\"/cgi-bin/index.cgi\" name=\"unused\" method=GET >"
        echo "<table width=\"99%\"  class=\"form_2\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">"
        
  	echo "<tr id=\"title\">"
    	echo "<th colspan=3>"
	echo "<label><span class=\"style1\">Password Management</span></label>"
        echo "&nbsp;"
    	echo "</th>"
    	
        echo "<tr>"
        echo "<td width=\"30%\"><div align=\"center\">"
        echo "$MESSAGE"
        echo "</div></td>"
        echo "</tr><tr><BR>"
        echo "</tr><tr>"
        echo "<td width=\"30%\"><div align=\"center\">"
        echo "<input type=\"submit\" name=\"action\" value=\"Return\">"
        echo "</div></td>"
        echo "</tr></table></form>"
}	


